//@author: vux
//@help: template for standard shaders
//@tags: template
//@credits:


      //the models world matrix
float4x4 tV: VIEW;         //view matrix as set via Renderer (EX9)
float4x4 tP: PROJECTION;
float4x4 tWVP: WORLDVIEWPROJECTION;
float4x4 tVP : LAYERVIEWPROJECTION;
float4x4 tW : WORLD;

static const float2 filters = float2(.5, .9);
float epsilon;
int _LinearColorSpace;
float4 TexelSize = float4(0.0019,0.0023,512,424) ;
int MeshDensity = 128;
float4 Crop   = float4 (0.22,0.321,0.5968, 0.3686);
float FarClip ;
float NearClip ;
float2 ImageDimensions ;
float2 PrincipalPoint ;
float2 depthFocalLength;
float4x4 extrinsics;
float multer;
float2 adder;


Texture2D mainTex <string uiname="Texture";>;

SamplerState linearSampler : IMMUTABLE
{
	Filter = MIN_MAG_MIP_LINEAR;
	AddressU = WRAP;
	AddressV = WRAP;
};


float3 rgb2hsv(float3 c)
{
	float4 K = float4(0.0, -1.0 / 3.0, 2.0 / 3.0, -1.0);
	float4 p = lerp(float4(c.bg, K.wz), float4(c.gb, K.xy), step(c.b, c.g));
	float4 q = lerp(float4(p.xyw, c.r), float4(c.r, p.yzx), step(p.x, c.r));
	
	float d = q.x - min(q.w, q.y);
	return float3(abs(q.z + (q.w - q.y) / (6.0 * d + epsilon)), d / (q.x + epsilon), q.x);
}

float depthForPoint(float2 texturePoint){
	
	float4 textureSample = float4( texturePoint.x, texturePoint.y, 0.0, 0.0);
	//float4 depthsample = pow( tex2Dlod(Samp, textureSample), _LinearColorSpace == 1 ? 0.4545 : 1.0);
	//this is akward
	float4 depthsample = pow ( abs(mainTex.SampleLevel(linearSampler, textureSample.xy,textureSample.z )), _LinearColorSpace == 1 ? 0.4545 : 1.0 );
	float3 depthsamplehsv = rgb2hsv(depthsample.rgb);
	
	return depthsamplehsv.g > filters.x && depthsamplehsv.b > filters.y ? depthsamplehsv.r : 0.0;
}

struct vsInput
{
	float4 PosO  : POSITION;
   float2 ColTexCd : TEXCOORD0;
	float2 DepTexCd : NORMAL;
	 //float3 valid  : TEXCOORD1;
};


struct vs2ps
{
	float4 Pos  : SV_POSITION;
	float2 ColTexCd : TEXCOORD0;
	float2 DepTexCd : NORMAL;
//	float valid  : TEXCOORD3;
};


vs2ps VS(vsInput input)
{
	vs2ps output;
	
	float4 vertIn = input.PosO;
	float2 depthTexCoord = input.DepTexCd;
	float2 colorTexCoord = input.ColTexCd;
	
	float2 centerpix = TexelSize.xy * 0.5 + adder  ;
	float2 textureStep = float2( 1.0 / (MeshDensity - 1.0), 1.0 / (MeshDensity - 1.0) );
    float2 basetex = floor(vertIn.xy * TexelSize.zw) * TexelSize.xy;
	
	depthTexCoord = multer* basetex * float2(1.0, 0.5) + centerpix;
	colorTexCoord = multer* basetex * float2(1.0, 0.5) + float2(0.0, 0.5) + centerpix;

		//check neighbors
		float2 neighbors[8] = {
		float2(			   0.,  textureStep.y),
		float2( textureStep.x,	 		   0.),
		float2(			   0., -textureStep.y),
		float2(-textureStep.x,			   0.),
		float2(-textureStep.x, -textureStep.y),
		float2( textureStep.x,  textureStep.y),
		float2( textureStep.x, -textureStep.y),
		float2(-textureStep.x,  textureStep.y)
	};
	
 	//texture coords come in as [0.0 - 1.0] for this whole plane
	float depth = depthForPoint(depthTexCoord);
    
	int i;
	float neighborDepths[8];
	for (i = 0; i < 8; i++) {
		neighborDepths[i] = depthForPoint(depthTexCoord + neighbors[i]);
	}

	//den exw idea ti einai auto
	//float valium = input.valid;
	float valium = 1.0;
	
	int numDudNeighbors = 0;
	//search neighbor verts in order to see if we are near an edge
	//if so, clamp to the surface closest to us
	if(depth < epsilon || (1.0 - depth) < epsilon){
		float depthDif = 1.0f;
		float nearestDepth = depth;
		for(int i = 0; i < 8; i++){
			//float depthNeighbor = depthForPoint(depthTexCoord + neighbors[i]);
			float depthNeighbor = neighborDepths[i];
			if(depthNeighbor >= epsilon && (1.0 - depthNeighbor) > epsilon){
				float thisDif = abs(nearestDepth - depthNeighbor);
				if(thisDif < depthDif){
					depthDif = thisDif;
					nearestDepth = depthNeighbor;
				}
			}
			else {
				numDudNeighbors++;
			}
		}

		depth = nearestDepth;
		
		if (depth < epsilon || (1.0-depth) < epsilon) {
			valium = 0.0;
		}
		if(numDudNeighbors > 6) {
			valium = 0.0;
		}
	}

		float2 imageCoordinates = Crop.xy + (basetex * Crop.zw);
	
		float z = depth * (FarClip - NearClip) + NearClip;
	
	float4 vertOut = float4(  ( imageCoordinates.x * ImageDimensions.x - PrincipalPoint.x )  * z / depthFocalLength.x,
	(imageCoordinates.y * ImageDimensions.y - PrincipalPoint.y ) * z /depthFocalLength.y , z , vertIn.w );
	
	vertOut = mul(extrinsics,vertOut);

    output.Pos = mul (vertOut,mul(tW,tVP ));

 	 output.ColTexCd = colorTexCoord;

output.DepTexCd = depthTexCoord;
//	output.valid = valium;
    return output;
}


float4 PS(vs2ps In): SV_Target
{ 
   float4 col = mainTex.Sample(linearSampler, In.ColTexCd);
	float3 depth = mainTex.Sample ( linearSampler, In.DepTexCd).rgb;
    float3 depthhsv = rgb2hsv(depth);
col.a = depthhsv.g * depthhsv.b ;
//	In.valid *= (depthhsv.r > epsilon && depthhsv.g > filters.x && depthhsv.b > filters.y) ? 1.0 : 0.0;

    return col
	;
}





technique11 Constant
{
	pass P0
	{
		SetVertexShader( CompileShader( vs_4_0, VS() ) );
		SetPixelShader( CompileShader( ps_4_0, PS() ) );
	}
}




